@@ -25,6 +25,7 @@
 #include <dirent.h>
 #include <signal.h>
 #include <unistd.h>
+#include <utmp.h>
 
 #include <locale.h>
 
@@ -33,7 +34,7 @@
 #endif
 
 #ifdef ENABLE_NOTIFY
-#include <sys/vfs.h>
+#include <sys/statvfs.h>
 #include <libnotify/notify.h>
 #endif
 
@@ -56,7 +57,7 @@
 
 #define warn(fmt,arg...) g_warning("%s/%d: " fmt,__FILE__,__LINE__,##arg)
 
-#define NAUTILUS_COMMAND	 BIN_NAUTILUS" -n --no-desktop %m"
+#define NAUTILUS_COMMAND BIN_NAUTILUS " -n --no-desktop %m"
 
 static struct gvm_configuration config;
 static DBusConnection *dbus_connection;
@@ -86,19 +87,22 @@ static guint statfs_id = 0;
 
 typedef struct {
         gboolean notified;
-        double last_pct;
+        double last_notified;
 	char *udi;
 } statfs_mount_info;
 
+#ifdef ENABLE_NOTIFY
 static void statfs_mount_info_add (const char *udi);
+static void statfs_mount_info_remove (const char *udi);
 static void statfs_mount_info_free (statfs_mount_info *info);
 static gboolean gvm_statfs_check_space (const char *udi, statfs_mount_info *info, gpointer user_data);
+#endif
 
 static gboolean gvm_dbus_init (void);
 static LibHalContext *gvm_hal_init (void);
 
-static gboolean gvm_user_at_console (void);
-static gboolean gvm_user_active_at_console (void);
+static gboolean gvm_local_user (void);
+static gboolean gvm_active_user (void);
 
 
 typedef enum {
@@ -116,6 +120,8 @@ enum {
 	AUTOIPOD_COMMAND,
 	AUTOMOUNT_DRIVES,
 	AUTOMOUNT_MEDIA,
+	AUTOOPEN,
+	AUTOOPEN_PATH,
 	AUTOPHOTO,
 	AUTOPHOTO_COMMAND,
 	AUTOPILOT,
@@ -156,6 +162,8 @@ static struct {
 	{ GCONF_ROOT "automount_media",           TYPE_BOOL,   &config.automount_media           },
 	{ GCONF_ROOT "automouse",                 TYPE_BOOL,   &config.automouse                 },
 	{ GCONF_ROOT "automouse_command",         TYPE_STRING, &config.automouse_command         },
+	{ GCONF_ROOT "autoopen",                  TYPE_BOOL,   &config.autoopen                  },
+	{ GCONF_ROOT "autoopen_path",             TYPE_STRING, &config.autoopen_path             },
 	{ GCONF_ROOT "autophoto",                 TYPE_BOOL,   &config.autophoto                 },
 	{ GCONF_ROOT "autophoto_command",         TYPE_STRING, &config.autophoto_command         },
 	{ GCONF_ROOT "autopalmsync",              TYPE_BOOL,   &config.autopilot                 },
@@ -175,7 +183,6 @@ static struct {
 	{ GCONF_ROOT "autotablet",                TYPE_BOOL,   &config.autotablet                },
 	{ GCONF_ROOT "autotablet_command",        TYPE_STRING, &config.autotablet_command        },
 	{ GCONF_ROOT "percent_threshold",         TYPE_FLOAT,  &config.percent_threshold         },
-	{ GCONF_ROOT "percent_freed",             TYPE_FLOAT,  &config.percent_freed             },
 	{ GCONF_ROOT "percent_used",              TYPE_FLOAT,  &config.percent_used              },
 };
 
@@ -191,12 +198,13 @@ struct _GvmPromptButton {
 enum {
 	GVM_RESPONSE_NONE,
 	GVM_RESPONSE_RUN,
+	GVM_RESPONSE_OPEN,
 	GVM_RESPONSE_PLAY,
 	GVM_RESPONSE_BROWSE,
 	GVM_RESPONSE_SYNC_MUSIC,
 	GVM_RESPONSE_IMPORT_PHOTOS,
-	GVM_RESPONSE_BURN_AUDIO_CD,
-	GVM_RESPONSE_BURN_DATA_CD,
+	GVM_RESPONSE_WRITE_AUDIO_CD,
+	GVM_RESPONSE_WRITE_DATA_CD,
 };
 
 static struct _GvmPromptButton GVM_BUTTONS_AUTORUN[] = {
@@ -204,6 +212,11 @@ static struct _GvmPromptButton GVM_BUTTONS_AUTORUN[] = {
 	{ N_("_Allow Auto-Run"), NULL, GVM_RESPONSE_RUN },
 };
 
+static struct _GvmPromptButton GVM_BUTTONS_AUTOOPEN[] = {
+	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("_Open"), NULL, GVM_RESPONSE_OPEN },
+};
+
 static struct _GvmPromptButton GVM_BUTTONS_CAMERA[] = {
 	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
@@ -227,26 +240,27 @@ static struct _GvmPromptButton GVM_BUTTONS_MIXED_CD[] = {
 	{ N_("_Play CD"), NULL, GVM_RESPONSE_PLAY },
 };
 
-static struct _GvmPromptButton GVM_BUTTONS_BURN_CD[] = {
+static struct _GvmPromptButton GVM_BUTTONS_WRITE_CD[] = {
 	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
-	{ N_("Make _Audio CD"), NULL, GVM_RESPONSE_BURN_AUDIO_CD },
-	{ N_("Make _Data CD"), NULL, GVM_RESPONSE_BURN_DATA_CD },
+	{ N_("Make _Audio CD"), NULL, GVM_RESPONSE_WRITE_AUDIO_CD },
+	{ N_("Make _Data CD"), NULL, GVM_RESPONSE_WRITE_DATA_CD },
 };
 
-static struct _GvmPromptButton GVM_BUTTONS_BURN_DVD[] = {
+static struct _GvmPromptButton GVM_BUTTONS_WRITE_DVD[] = {
 	{ N_("Ig_nore"), NULL, GTK_RESPONSE_CANCEL },
-	{ N_("Make _DVD"), NULL, GVM_RESPONSE_BURN_DATA_CD },
+	{ N_("Make _DVD"), NULL, GVM_RESPONSE_WRITE_DATA_CD },
 };
 
 typedef enum {
 	GVM_PROMPT_AUTORUN,
+	GVM_PROMPT_AUTOOPEN,
 	GVM_PROMPT_IMPORT_CAMERA,
 	GVM_PROMPT_IMPORT_STORAGE_CAMERA,
 	GVM_PROMPT_IMPORT_PHOTOS,
 	GVM_PROMPT_IPOD_PHOTO,
 	GVM_PROMPT_CDA_EXTRA,
-	GVM_PROMPT_BURN_CDR,
-	GVM_PROMPT_BURN_DVD,
+	GVM_PROMPT_WRITE_CDR,
+	GVM_PROMPT_WRITE_DVD,
 } GvmPrompt;
 
 typedef struct _GvmPromptCtx GvmPromptCtx;
@@ -291,6 +305,13 @@ static struct {
 	  N_("Auto-run capability detected."),
 	  N_("Would you like to allow <b>'${0}'</b> to run?"), TRUE, TRUE,
 	  NULL, NULL },
+	{ 0, "gnome-fs-executable", NULL,
+	  GVM_BUTTONS_AUTOOPEN, G_N_ELEMENTS (GVM_BUTTONS_AUTOOPEN),
+	  GVM_RESPONSE_RUN,
+	  N_("Auto-Open Confirmation"),
+	  N_("Auto-Open capability detected."),
+	  N_("Would you like to open <b>'${0}'</b>?"), TRUE, TRUE,
+	  NULL, NULL },
 	{ 0, "applets-screenshooter", NULL,
 	  GVM_BUTTONS_CAMERA, G_N_ELEMENTS (GVM_BUTTONS_CAMERA),
 	  GVM_RESPONSE_IMPORT_PHOTOS,
@@ -327,15 +348,15 @@ static struct {
 	  N_("Would you like to listen to music or browse the files?"), FALSE, FALSE,
 	  GCONF_ROOT "prompts/cd_mixed", N_("_Always perform this action") },
 	{ 0, "gnome-dev-cdrom", NULL,
-	  GVM_BUTTONS_BURN_CD, G_N_ELEMENTS (GVM_BUTTONS_BURN_CD),
-	  GVM_RESPONSE_BURN_DATA_CD,
+	  GVM_BUTTONS_WRITE_CD, G_N_ELEMENTS (GVM_BUTTONS_WRITE_CD),
+	  GVM_RESPONSE_WRITE_DATA_CD,
 	  N_("Choose Disc Type"),
 	  N_("You have inserted a blank disc."),
 	  N_("What would you like to do?"), FALSE, FALSE,
 	  NULL, NULL },
 	{ 0, "gnome-dev-disc-dvdr", NULL,
-	  GVM_BUTTONS_BURN_DVD, G_N_ELEMENTS (GVM_BUTTONS_BURN_DVD),
-	  GVM_RESPONSE_BURN_DATA_CD,
+	  GVM_BUTTONS_WRITE_DVD, G_N_ELEMENTS (GVM_BUTTONS_WRITE_DVD),
+	  GVM_RESPONSE_WRITE_DATA_CD,
 	  N_("Choose Disc Type"),
 	  N_("You have inserted a blank disc."),
 	  N_("What would you like to do?"), FALSE, FALSE,
@@ -1162,7 +1183,7 @@ autorun_cb (GvmPromptCtx *ctx, int action)
 		argv[0] = ctx->path;
 		argv[1] = NULL;
 		
-		g_spawn_async (g_get_home_dir (), argv, NULL,
+		g_spawn_async (ctx->mount_point, argv, NULL,
 			       0, NULL, NULL, NULL, &error);
 		if (error)
 			warn ("failed to exec %s: %s", ctx->path, error->message);
@@ -1174,6 +1195,74 @@ autorun_cb (GvmPromptCtx *ctx, int action)
 		gvm_run_command (NAUTILUS_COMMAND, ctx->udi, ctx->device, ctx->mount_point);
 }
 
+static void
+autoopen_cb (GvmPromptCtx *ctx, int action)
+{
+	gboolean autobrowse = TRUE;
+	GError *error = NULL;
+	char *argv[3];
+	
+	if (action == GVM_RESPONSE_OPEN) {
+		argv[0] = "gnome-open";
+		argv[1] = ctx->path;
+		argv[2] = NULL;
+		
+		g_spawn_async (ctx->mount_point, argv, NULL,
+			       0, NULL, NULL, NULL, &error);
+		if (error)
+			warn ("failed to exec %s: %s", ctx->path, error->message);
+		else
+			autobrowse = FALSE;
+	}
+	
+	if (config.autobrowse && autobrowse)
+		gvm_run_command (NAUTILUS_COMMAND, ctx->udi, ctx->device, ctx->mount_point);
+}
+
+static char *
+canon_path (char *path)
+{
+	register char *inptr, *outptr;
+	
+	inptr = outptr = path;
+	
+	while (*inptr != '\0') {
+		if (inptr[0] == '.' && (inptr[1] == '/' || (inptr[1] == '.' && inptr[2] == '/'))) {
+			if (inptr[1] == '.') {
+				/* ../ */
+				inptr += 3;
+				
+				if (outptr > path) {
+					outptr--;
+					while (outptr > path && outptr[-1] != '/')
+						outptr--;
+					if (outptr == path && *outptr == '/')
+						outptr++;
+				}
+			} else {
+				/* ./ */
+				inptr += 2;
+			}
+			
+			while (*inptr == '/')
+				inptr++;
+		}
+		
+		while (*inptr && *inptr != '/')
+			*outptr++ = *inptr++;
+		
+		if (*inptr == '/')
+			*outptr++ = *inptr++;
+		
+		while (*inptr == '/')
+			inptr++;
+	}
+	
+	*outptr = '\0';
+	
+	return path;
+}
+
 /*
  * gvm_autorun - automatically execute stuff
  *
@@ -1182,8 +1271,10 @@ autorun_cb (GvmPromptCtx *ctx, int action)
 static void
 gvm_autorun (const char *udi, const char *device, const char *mount_point)
 {
-	gboolean autobrowse = TRUE;
+	char **autopath, *argv[1], *path;
+	gboolean handled = FALSE;
 	GvmPromptCtx *ctx;
+	int i;
 	
 	if (gvm_check_dvd (udi, device, mount_point))
 		return;
@@ -1198,33 +1289,104 @@ gvm_autorun (const char *udi, const char *device, const char *mount_point)
 	}
 	
 	if (config.autorun && config.autorun_path) {
-		char **autorun;
-		int i;
+		autopath = g_strsplit (config.autorun_path, ":", -1);
 		
-		autorun = g_strsplit (config.autorun_path, ":", -1);
-		
-		for (i = 0; autorun[i]; i++) {
-			char *argv[1];
-			char *path;
-			
-			path = g_strdup_printf ("%s/%s", mount_point, autorun[i]);
+		for (i = 0; autopath[i]; i++) {
+			path = g_strdup_printf ("%s/%s", mount_point, autopath[i]);
 			if (access (path, F_OK | R_OK | X_OK) != 0) {
 				g_free (path);
 				continue;
 			}
 			
-			argv[0] = autorun[i];
+			argv[0] = autopath[i];
 			ctx = gvm_prompt_ctx_new (GVM_PROMPT_AUTORUN, autorun_cb, udi, device, mount_point, path);
 			gvm_prompt (ctx, 1, argv);
-			autobrowse = FALSE;
+			handled = TRUE;
 			g_free (path);
 			break;
 		}
 		
-		g_strfreev (autorun);
+		g_strfreev (autopath);
+		
+		if (handled)
+			return;
 	}
 	
-	if (config.autobrowse && autobrowse)
+	if (config.autoopen && config.autoopen_path) {
+		autopath = g_strsplit (config.autoopen_path, ":", -1);
+		
+		for (i = 0; autopath[i]; i++) {
+			char *outbuf = NULL, *outptr = NULL, buf[256];
+			gboolean eoln = FALSE;
+			size_t n, m;
+			FILE *fp;
+			
+			path = g_strdup_printf ("%s/%s", mount_point, autopath[i]);
+			if (!(fp = fopen (path, "rt"))) {
+				g_free (path);
+				continue;
+			}
+			
+			g_free (path);
+			
+			while (!eoln && fgets (buf, sizeof (buf), fp)) {
+				n = strlen (buf);
+				
+				if (buf[n - 1] == '\n') {
+					eoln = TRUE;
+					buf[--n] = '\0';
+					if (n > 0 && buf[n - 1] == '\r')
+						buf[--n] = '\0';
+				}
+				
+				if (outbuf || !eoln) {
+					if (outbuf) {
+						m = outptr - outbuf;
+						outbuf = g_realloc (outbuf, m + n + 1);
+						outptr = g_stpcpy (outbuf + m, buf);
+					} else {
+						outbuf = g_malloc (n + 1);
+						outptr = g_stpcpy (outbuf, buf);
+					}
+					
+					path = outbuf;
+				} else {
+					path = buf;
+				}
+			}
+			
+			fclose (fp);
+			
+			if (path[0] == '/') {
+				/* not allowed */
+				g_free (outbuf);
+				break;
+			}
+			
+			path = g_build_filename (mount_point, path, NULL);
+			g_free (outbuf);
+			
+			if (strncmp (canon_path (path), mount_point, strlen (mount_point) != 0)) {
+				/* not allowed */
+				g_free (path);
+				break;
+			}
+			
+			argv[0] = autopath[i];
+			ctx = gvm_prompt_ctx_new (GVM_PROMPT_AUTOOPEN, autoopen_cb, udi, device, mount_point, path);
+			gvm_prompt (ctx, 1, argv);
+			handled = TRUE;
+			g_free (path);
+			break;
+		}
+		
+		g_strfreev (autopath);
+		
+		if (handled)
+			return;
+	}
+	
+	if (config.autobrowse)
 		gvm_run_command (NAUTILUS_COMMAND, udi, device, mount_point);
 }
 
@@ -1345,23 +1507,25 @@ gvm_device_mounted (const char *udi)
 		ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_STORAGE_CAMERA, import_storage_camera_cb, udi, device, mount_point, NULL);
 		gvm_prompt (ctx, 0, NULL);
 	} else if (config.autoipod && gvm_udi_is_portable_media_player (udi, &is_ipod)) {
+		char *ipod_control = NULL;
+		const char *photo_dir;
+		
 		if (is_ipod) {
-			char *ipod_control;
-			
 			ipod_control = g_build_filename (mount_point, "iPod_Control", NULL);
-			
-			if (gvm_check_photos (udi, device, ipod_control)) {
-				/* we have ourselves an iPod Photo - need to prompt what to do */
-				ctx = gvm_prompt_ctx_new (GVM_PROMPT_IPOD_PHOTO, ipod_photo_cb, udi, device, mount_point, ipod_control);
-				gvm_prompt (ctx, 0, NULL);
-			} else {
-				gvm_run_portable_media_player (udi, device, mount_point);
-			}
-			
-			g_free (ipod_control);
+			photo_dir = ipod_control;
+		} else {
+			photo_dir = mount_point;
+		}
+		
+		if (gvm_check_photos (udi, device, photo_dir)) {
+			/* we have ourselves an iPod Photo - need to prompt what to do */
+			ctx = gvm_prompt_ctx_new (GVM_PROMPT_IPOD_PHOTO, ipod_photo_cb, udi, device, mount_point, photo_dir);
+			gvm_prompt (ctx, 0, NULL);
 		} else {
 			gvm_run_portable_media_player (udi, device, mount_point);
 		}
+		
+		g_free (ipod_control);
 	} else {
 		gvm_autorun (udi, device, mount_point);
 	}
@@ -1378,19 +1542,20 @@ enum {
 	MOUNT_DATA       = (1 << 1),
 	MOUNT_DIRSYNC    = (1 << 2),
 	MOUNT_DMASK      = (1 << 3),
-	MOUNT_FLUSH      = (1 << 4),
-	MOUNT_IOCHARSET  = (1 << 5),
-	MOUNT_MODE       = (1 << 6),
-	MOUNT_NOATIME    = (1 << 7),
-	MOUNT_NODIRATIME = (1 << 8),
-	MOUNT_NOEXEC     = (1 << 9),
-	MOUNT_QUIET      = (1 << 10),
-	MOUNT_READ_ONLY  = (1 << 11),
-	MOUNT_SHORTNAME  = (1 << 12),
-	MOUNT_SYNC       = (1 << 13),
-	MOUNT_UID        = (1 << 14),
-	MOUNT_UMASK      = (1 << 15),
-	MOUNT_UTF8       = (1 << 16),
+	MOUNT_FMASK      = (1 << 4),
+	MOUNT_FLUSH      = (1 << 5),
+	MOUNT_IOCHARSET  = (1 << 6),
+	MOUNT_MODE       = (1 << 7),
+	MOUNT_NOATIME    = (1 << 8),
+	MOUNT_NODIRATIME = (1 << 9),
+	MOUNT_NOEXEC     = (1 << 10),
+	MOUNT_QUIET      = (1 << 11),
+	MOUNT_READ_ONLY  = (1 << 12),
+	MOUNT_SHORTNAME  = (1 << 13),
+	MOUNT_SYNC       = (1 << 14),
+	MOUNT_UID        = (1 << 15),
+	MOUNT_UMASK      = (1 << 16),
+	MOUNT_UTF8       = (1 << 17),
 };
 
 static struct {
@@ -1400,7 +1565,8 @@ static struct {
 	{ "codepage=",  MOUNT_CODEPAGE   },  /* vfat */
 	{ "data=",      MOUNT_DATA       },  /* ext3 */
 	{ "dirsync",    MOUNT_DIRSYNC    },
-	{ "dmask=",     MOUNT_DMASK      },  /* ntfs */
+	{ "dmask=",     MOUNT_DMASK      },  /* vfat, ntfs */
+	{ "fmask=",     MOUNT_FMASK      },  /* vfat, ntfs */
 	{ "flush",      MOUNT_FLUSH      },  /* vfat */
 	{ "iocharset=", MOUNT_IOCHARSET  },  /* vfat, iso9660 */
 	{ "mode=",      MOUNT_MODE       },  /* iso9660 */
@@ -1595,7 +1761,7 @@ gvm_device_mount (const char *udi, gboolean interactive)
 		
 		return retval;
 	} else {
-		char *mount_point, *fstype, *charset_opt = NULL;
+		char *mount_point, *fstype, fmask_opt[12], *charset_opt = NULL;
 		DBusMessage *dmesg, *reply;
 		GPtrArray *options;
 		DBusError error;
@@ -1614,6 +1780,7 @@ gvm_device_mount (const char *udi, gboolean interactive)
 			guint32 opts = 0;
 			char uid[32];
 			size_t i, j;
+			mode_t mask;
 			
 			if ((moptions = libhal_device_get_property_strlist (hal_ctx, udi, "volume.mount.valid_options", NULL))) {
 				for (i = 0; moptions[i]; i++) {
@@ -1627,6 +1794,9 @@ gvm_device_mount (const char *udi, gboolean interactive)
 			}
 			
 			if (!strcmp (fstype, "vfat")) {
+				if (opts & MOUNT_NOEXEC)
+					g_ptr_array_add (options, "noexec");
+				
 				/* preferably mount flush (opportunistic syncing) if available, else
 				 * mount sync for "small" volumes (that are likely to be thumb drives)
 				 */
@@ -1640,12 +1810,21 @@ gvm_device_mount (const char *udi, gboolean interactive)
 						g_ptr_array_add (options, "sync");
 				}
 				
+				if (opts & MOUNT_FMASK) {
+					mask = umask (0);
+					snprintf (fmask_opt, sizeof (fmask_opt), "fmask=%#o", mask | 0111);
+					g_ptr_array_add (options, fmask_opt);
+					umask (mask);
+				}
+				
 				if (opts & MOUNT_SHORTNAME)
-					g_ptr_array_add (options, "shortname=winnt");
+					g_ptr_array_add (options, "shortname=lower");
 			} else if (!strcmp (fstype, "iso9660")) {
 				/* only care about uid= and iocharset= */
 			} else if (!strcmp (fstype, "udf")) {
-				/* only care about uid= and iocharset= */
+				/* also care about uid= and iocharset= */
+				if (opts & MOUNT_NOATIME)
+					g_ptr_array_add (options, "noatime");
 			}
 			
 			if (opts & (MOUNT_IOCHARSET|MOUNT_UTF8)) {
@@ -1886,10 +2065,10 @@ burn_cdr_cb (GvmPromptCtx *ctx, int action)
 	const char *command;
 	
 	switch (action) {
-	case GVM_RESPONSE_BURN_AUDIO_CD:
+	case GVM_RESPONSE_WRITE_AUDIO_CD:
 		command = config.autoburn_audio_cd_command;
 		break;
-	case GVM_RESPONSE_BURN_DATA_CD:
+	case GVM_RESPONSE_WRITE_DATA_CD:
 		command = config.autoburn_data_cd_command;
 		break;
 	default:
@@ -1913,9 +2092,9 @@ gvm_run_cdburner (const char *udi, int type, const char *device, const char *mou
 		return;
 	
 	if (type == WRITER_TYPE_DVD)
-		prompt = GVM_PROMPT_BURN_DVD;
+		prompt = GVM_PROMPT_WRITE_DVD;
 	else
-		prompt = GVM_PROMPT_BURN_CDR;
+		prompt = GVM_PROMPT_WRITE_CDR;
 	
 	ctx = gvm_prompt_ctx_new (prompt, burn_cdr_cb, udi, device, mount_point, NULL);
 	gvm_prompt (ctx, 0, NULL);
@@ -1931,33 +2110,33 @@ gvm_cdrom_policy (const char *udi)
 {
 	dbus_bool_t has_audio;
 	dbus_bool_t has_data;
-	dbus_bool_t is_blank;
 	char *device = NULL;
 	DBusError error;
-	int type;
+	writer_t type;
 	
 	dbus_error_init (&error);
-	has_audio = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_audio", NULL);
-	has_data = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_data", NULL);
-	is_blank = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.is_blank", NULL);
-	
 	if (!(device = libhal_device_get_property_string (hal_ctx, udi, "block.device", &error))) {
 		warn ("cannot get block.device: %s", error.message);
 		dbus_error_free (&error);
 		return;
 	}
 	
-	if (has_audio && !has_data) {
-		if (config.autoplay_cda)
-			gvm_run_cdplayer (udi, device, NULL);
-	} else if (has_audio && has_data) {
-		gvm_ask_mixed (udi);
-	} else if (has_data) {
-		if (config.automount_media && !gvm_udi_is_subfs_mount (udi))
-			gvm_device_mount (udi, TRUE);
-	} else if (is_blank) {
+	if (libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.is_blank", NULL)) {
 		if ((type = gvm_cdrom_media_is_writable (udi)))
 			gvm_run_cdburner (udi, type, device, NULL);
+	} else {
+		has_audio = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_audio", NULL);
+		has_data = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_data", NULL);
+		
+		if (has_audio && has_data) {
+			gvm_ask_mixed (udi);
+		} else if (has_audio) {
+			if (config.autoplay_cda)
+				gvm_run_cdplayer (udi, device, NULL);
+		} else if (has_data) {
+			if (config.automount_media && !gvm_udi_is_subfs_mount (udi))
+				gvm_device_mount (udi, TRUE);
+		}
 	}
 	
 	/** @todo enforce policy for all the new disc types now supported */
@@ -2001,34 +2180,6 @@ gvm_media_changed (const char *udi, const char *storage_device)
 }
 
 
-/* Return whether or not the device should be automounted based on the
- * HAL automount hint. Default to returning TRUE if the property isn't
- * set for the device or its parent storage device. */
-static gboolean
-gvm_automount_enabled (const char *udi)
-{
-	gboolean enabled = TRUE;
-	DBusError error;
-	char *drive;
-	
-	if (libhal_device_property_exists (hal_ctx, udi, "storage.automount_enabled_hint", NULL))
-		return libhal_device_get_property_bool (hal_ctx, udi, "storage.automount_enabled_hint", NULL);
-	
-	dbus_error_init (&error);
-	if (!(drive = libhal_device_get_property_string (hal_ctx, udi, "info.parent", &error))) {
-		warn ("couldn't get info.parent for %s: %s", udi, error.message);
-		dbus_error_free (&error);
-		return TRUE; /* sensible default? */
-	}
-	
-	if (libhal_device_property_exists (hal_ctx, drive, "storage.automount_enabled_hint", NULL))
-		enabled = libhal_device_get_property_bool (hal_ctx, drive, "storage.automount_enabled_hint", NULL);
-	
-	libhal_free_string (drive);
-	
-	return enabled;
-}
-
 static void
 import_camera_cb (GvmPromptCtx *ctx, int action)
 {
@@ -2092,6 +2243,14 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 		goto out;
 	}
 	
+	/* if the partition_table_changed flag is set, we don't want
+	 * to mount as a partitioning tool might be modifying this
+	 * device */
+	if (libhal_device_get_property_bool (hal_ctx, storage_device, "storage.partition_table_changed", NULL)) {
+		dbg ("partition table changed for %s\n", storage_device);
+		goto out;
+	}
+	
 	/*
 	 * Does this device support removable media?  Note that we
 	 * check storage_device and not our own UDI
@@ -2105,10 +2264,10 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 	
 	if (config.automount_drives && (mountable || crypto)) {
 		if (!gvm_udi_is_subfs_mount (udi)) {
-			if (gvm_automount_enabled (udi)) {
+			if (!libhal_device_get_property_bool (hal_ctx, udi, "volume.ignore", NULL)) {
 				gvm_device_mount (udi, TRUE);
 			} else {
-				dbg ("storage.automount_enabled_hint set to false on %s, not mounting\n", udi);
+				dbg ("volume.ignore set to true on %s, not mounting\n", udi);
 			}
 		}
 	}
@@ -2282,7 +2441,7 @@ hal_device_added (LibHalContext *ctx __attribute__((__unused__)),
 	char **capabilities;
 	size_t i, j, n;
 	
-	if (!gvm_user_active_at_console ())
+	if (!gvm_active_user ())
 		return;
 	
 	dbg ("Device added: %s\n", udi);
@@ -2391,31 +2550,20 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 			
 			g_free (policy->udi);
 			g_free (policy);
-		} else if (gvm_user_active_at_console ()) {
+		} else if (gvm_active_user ()) {
 			dbg ("not in mount queue: %s\n", udi);
-			gvm_device_mounted (udi);
+			/*gvm_device_mounted (udi);*/
 		}
 		
 #ifdef ENABLE_NOTIFY
-		/* If we aren't a cdrom, then add to the statfs list */
-		if (!gvm_udi_is_cdrom (udi)) {
-			if (!g_hash_table_lookup (statfs_mounts, udi))
-				statfs_mount_info_add (udi);
-		}
+		statfs_mount_info_add (udi);
 #endif
 	} else {
 		dbg ("Unmounted: %s\n", udi);
 		
 #ifdef ENABLE_NOTIFY
 		/* remove the udi from the statfs_mounts list */
-		if (!gvm_udi_is_cdrom (udi)) {
-			statfs_mount_info *info;
-			
-                        if ((info = g_hash_table_lookup (statfs_mounts, udi))) {
-				g_hash_table_remove (statfs_mounts, udi);
-				statfs_mount_info_free (info);
-			}
-		}
+		statfs_mount_info_remove (udi);
 #endif
 		
 		/* unmount all volumes mounted during lifetime */
@@ -2522,7 +2670,7 @@ hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
 		      const char *udi, const char *condition_name,
 		      const char *condition_details __attribute__((__unused__)))
 {
-	if (!gvm_user_active_at_console ())
+	if (!gvm_active_user ())
 		return;
 	
 	if (!strcmp (condition_name, "EjectPressed"))
@@ -2533,6 +2681,7 @@ static gboolean
 reinit_dbus (gpointer user_data)
 {
 	if (gvm_dbus_init ()) {
+		/* dbus daemon is up, get a hal context */
 		if ((hal_ctx = gvm_hal_init ()))
 			libhal_ctx_set_dbus_connection (hal_ctx, dbus_connection);
 		else
@@ -2541,6 +2690,8 @@ reinit_dbus (gpointer user_data)
 		return FALSE;
 	}
 	
+	/* dbus deamon not back up yet */
+	
 	return TRUE;
 }
 
@@ -2568,6 +2719,9 @@ gvm_dbus_init (void)
 {
 	DBusError error;
 	
+	if (dbus_connection != NULL)
+		return TRUE;
+	
 	dbus_error_init (&error);
 	if (!(dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error))) {
 		dbg ("could not get system bus: %s\n", error.message);
@@ -2678,11 +2832,7 @@ mount_all (LibHalContext *ctx)
 
 		if (gvm_udi_is_subfs_mount (udi)) {
 #ifdef ENABLE_NOTIFY
-			/* Add to the statfs_mounts list if not a cdrom */
-			if (!gvm_udi_is_cdrom (udi)) {
-				if (!g_hash_table_lookup (statfs_mounts, udi))
-					statfs_mount_info_add (udi);
-			}
+			statfs_mount_info_add (udi);
 #endif
 			continue;
 		}
@@ -2691,11 +2841,7 @@ mount_all (LibHalContext *ctx)
 		if (!libhal_device_property_exists (ctx, udi, "volume.is_mounted", NULL)
 		    || libhal_device_get_property_bool (ctx, udi, "volume.is_mounted", NULL)) {
 #ifdef ENABLE_NOTIFY
-			/* Add to the statfs_mounts list if not a cdrom */
-			if (!gvm_udi_is_cdrom (udi)) {
-                                if (!g_hash_table_lookup (statfs_mounts, udi))
-					statfs_mount_info_add (udi);
-			}
+			statfs_mount_info_add (udi);
 #endif
 			continue;
 		}
@@ -2721,7 +2867,7 @@ mount_all (LibHalContext *ctx)
 			 && libhal_device_get_property_bool (ctx, drive, "storage.removable", NULL))
 			mount = config.automount_media;
 		else
-			mount = gvm_automount_enabled (udi);
+			mount = !libhal_device_get_property_bool (ctx, udi, "volume.ignore", NULL);
 		libhal_free_string (drive);
 		
 		if (!mount)
@@ -2789,7 +2935,7 @@ sigterm_iochn_data (GIOChannel *source,
 	GError *err = NULL;
 	gchar data[1];
 	gsize bytes_read;
-
+	
 	/* Empty the pipe */
 	if (G_IO_STATUS_NORMAL != g_io_channel_read_chars (source, data, 1, &bytes_read, &err)) {
 		warn ("Error emptying callout notify pipe: %s", err->message);
@@ -2801,13 +2947,6 @@ sigterm_iochn_data (GIOChannel *source,
 	
 	unmount_all ();
 	
-#ifdef ENABLE_NOTIFY
-	g_source_remove (statfs_id);
-	statfs_id = 0;
-	
-	g_hash_table_destroy (statfs_mounts);
-#endif
-
 	gtk_main_quit ();
 	
  out:
@@ -2815,21 +2954,43 @@ sigterm_iochn_data (GIOChannel *source,
 }
 
 
+#ifdef ENABLE_NOTIFY
 static void
 statfs_mount_info_add (const char *udi)
 {
 	statfs_mount_info *info;
 	
+	if (g_hash_table_lookup (statfs_mounts, udi))
+		return;
+	
+	if (libhal_device_get_property_bool (hal_ctx, udi, "volume.is_mounted_read_only", NULL))
+		return;
+	
+	/* FIXME: this check can probably be dropped */
+	if (gvm_udi_is_cdrom (udi))
+		return;
+	
 	info = g_new0 (statfs_mount_info, 1);
 	info->udi = g_strdup (udi);
 	info->notified = FALSE;
-	info->last_pct = 0.0;
+	info->last_notified = 0.0;
 	
 	g_hash_table_insert (statfs_mounts, info->udi, info);
 	
 	gvm_statfs_check_space (info->udi, info, NULL);
 }
 
+static void
+statfs_mount_info_remove (const char *udi)
+{
+	statfs_mount_info *info;
+	
+	if ((info = g_hash_table_lookup (statfs_mounts, udi))) {
+		g_hash_table_remove (statfs_mounts, udi);
+		statfs_mount_info_free (info);
+	}
+}
+
 static void
 statfs_mount_info_free (statfs_mount_info *info)
 {
@@ -2840,76 +3001,72 @@ statfs_mount_info_free (statfs_mount_info *info)
 static gboolean
 gvm_statfs_check_space (const char *udi, statfs_mount_info *info, gpointer user_data)
 {
-#ifdef ENABLE_NOTIFY
 	char *mount_point = NULL;
 	NotifyNotification *n;
-	struct statfs buf;
+	struct statvfs buf;
 	
 	if (!(mount_point = libhal_device_get_property_string (hal_ctx, udi, "volume.mount_point", NULL)))
 		return TRUE;
 	
-	if (statfs (mount_point, &buf) != -1) {
+	if (statvfs (mount_point, &buf) != -1) {
+		unsigned long twogb_blocks;
 		double free_space;
 		
 		free_space = (double) buf.f_bavail / (double) buf.f_blocks;
-		
-		if ((free_space - info->last_pct) > config.percent_used)
+		twogb_blocks = (2 * 1024 * 1024) / buf.f_bsize;
+
+		/*
+		   If we continue using up more and more space or if we pass
+		   the threshold value while running, we want to notify again
+		*/
+		if ((info->last_notified - free_space) > config.percent_used
+		    || (free_space < config.percent_threshold && info->last_notified > config.percent_threshold))
 			info->notified = FALSE;
-		else if ((free_space - info->last_pct) < config.percent_freed)
-			info->notified = TRUE;
-		
-		if (!info->notified && free_space < config.percent_threshold) {
-			char *low_disk_msg = NULL;
-			char *label = NULL;
+
+		/*
+		   If we didn't notify before (or need to again as above),
+		   and the %free is less than the threshold, and the amount of
+		   free space is less than 2GB, then notify the user
+		*/
+		if (!info->notified && (free_space < config.percent_threshold && buf.f_bavail < twogb_blocks)) {
+			char *disk, *label, *msg;
 			int free_pct;
 			
 			label = libhal_device_get_property_string (hal_ctx, udi, "volume.label", NULL);
-			if (!label || label[0] == '\0') {
-				libhal_free_string (label);
-				label = g_strdup (mount_point);
-			}
-			
-			free_pct = 100 - (free_space * 100);
-			low_disk_msg = g_strdup_printf (_("%d%% of the disk space on %s is in use"), free_pct, label);
-			
-			n = notify_notification_new ("Low Disk Space", low_disk_msg, "gnome-dev-harddisk", NULL);
-			notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
-			notify_notification_show (n, NULL);
-			
-			g_object_unref (n);
-			
-			libhal_free_string (low_disk_msg);
+			if (!label || label[0] == '\0')
+				disk = g_strdup (mount_point);
+			else
+				disk = g_strdup (label);
 			libhal_free_string (label);
 			
+			free_pct = 100 - (free_space * 100);
+			msg = g_strdup_printf (_("%d%% of the disk space on %s is in use"), free_pct, disk);
+			g_free (disk);
+			
+			n = notify_notification_new ("Low Disk Space", msg, "gnome-dev-harddisk", NULL);
+			notify_notification_set_urgency (n, NOTIFY_URGENCY_CRITICAL);
+			notify_notification_show (n, NULL);
+			g_object_unref (n);
+			g_free (msg);
+			
 			info->notified = TRUE;
+			info->last_notified = free_space;
 		}
-		
-		info->last_pct = free_space;
 	}
 	
 	libhal_free_string (mount_point);
 	
 	return TRUE;
-#else
-	udi = udi;
-	info = info;
-	user_data = user_data;
-	
-	return FALSE;
-#endif
 }
 
 static gboolean
 gvm_statfs_timeout (gpointer data)
 {
-#ifdef ENABLE_NOTIFY
 	g_hash_table_foreach (statfs_mounts, (GHFunc) gvm_statfs_check_space, NULL);
 	
 	return TRUE;
-#else
-	return FALSE;
-#endif
 }
+#endif /* ENABLE_NOTIFY */
 
 static void
 gvm_die (GnomeClient *client __attribute__((__unused__)),
@@ -2918,40 +3075,49 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 	dbg ("Received 'die', initiating shutdown\n");
 	
 	unmount_all ();
-
-#ifdef ENABLE_NOTIFY
-	g_source_remove (statfs_id);
-	statfs_id = 0;
-	
-	g_hash_table_destroy (statfs_mounts);
-#endif
 	
 	gtk_main_quit ();
 }
 
 
-/* checks that the user is logged-in at the console (which does not necessarily infer an *active* session) */
+/* checks that the user is logged-in at a local X session (which does not necessarily infer an *active* session) */
 static gboolean
-gvm_user_at_console (void)
+gvm_local_user (void)
 {
-	static unsigned int exists = 0;
-	char *path;
+	gboolean local = FALSE;
+	struct utmp *utmp;
+	const char *user;
+	size_t ulen;
 	
-	if (!(exists & 0x2)) {
-		path = g_build_filename (GVM_CONSOLE_AUTH_DIR, g_get_user_name (), NULL);
-		exists = (access (path, F_OK) == 0) ? 0x3 : 0x2;
-		g_free (path);
+	user = g_get_user_name ();
+	ulen = strlen (user);
+	
+	setutent ();
+	
+	while (!local && (utmp = getutent ())) {
+		if (utmp->ut_type != USER_PROCESS || strncmp (utmp->ut_user, user, ulen) != 0)
+			continue;
+		
+		/* only accept local X sessions */
+		local = utmp->ut_line[0] == ':' && utmp->ut_line[1] >= '0' && utmp->ut_line[1] <= '9';
 	}
 	
-	return (exists & 0x1);
+	endutent ();
+	
+	return local;
 }
 
-/* checks that the user's session is the active session at the console */
+/* checks that the user is at the local active X session */
 static gboolean
-gvm_user_active_at_console (void)
+gvm_active_user (void)
 {
+#ifdef ENABLE_MULTIUSER
 	/* FIXME: it is currently impossible to get this information */
 	return TRUE;
+#else
+	/* for the non-multiuser configuration, we assume we're always on the active console */
+	return TRUE;
+#endif
 }
 
 
@@ -3015,11 +3181,11 @@ main (int argc, char **argv)
 	}
 	
 	client = gnome_master_client ();
-	if (gvm_get_clipboard () && gvm_user_at_console ()) {
+	if (gvm_get_clipboard () && gvm_local_user ()) {
 		gnome_client_set_restart_style (client, GNOME_RESTART_ANYWAY);
 	} else {
 		gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-		if (gvm_user_at_console ())
+		if (gvm_local_user ())
 			warn ("already running");
 		
 		return 1;
@@ -3059,14 +3225,18 @@ main (int argc, char **argv)
 #endif
 
 	mount_all (hal_ctx);
-
+	
+#ifdef ENABLE_NOTIFY
 	/* Add a timeout for our statfs checking */
 	statfs_id = g_timeout_add (15000, (GSourceFunc) gvm_statfs_timeout, NULL);
+#endif
 	
 	gtk_main ();
 	
 #ifdef ENABLE_NOTIFY
 	g_hash_table_destroy (statfs_mounts);
+	g_source_remove (statfs_id);
+	statfs_id = 0;
 #endif
 	
 	g_hash_table_destroy (mount_table);
